﻿using UnityEngine;
using System.Collections;

public class Animator : MonoBehaviour
{
	public Renderer sprite;
	public Renderer magicSprite;
	public int frameDuration = 6;
	public int rampedUpFrameDuration = 5;
	public int spritesX = 4;
	public int spritesY = 2;
	
	
	private int framesSinceStart = 0;
	private int frame = 0;
	private Vector3 hornPos;
	
	/// <summary>
	/// 0 = running, 1 = ascending, 2 = neutral air, 3 = falling
	/// </summary>
	public int state = 0;
	public int chargeLevel = 0;
	
	
    void FixedUpdate ()
    {
		if(!MyLittleLevelManager.Get().bossFight )
			{
			// if ramp is reached, speed up hoofbeats
			int tempInt = frameDuration;
			if(MyLittleCamera.Get().transform.position.x > 7680)
			{
				tempInt--;
				if(MyLittleCamera.Get().transform.position.x > 7980)
				{
					tempInt--;
					if(MyLittleCamera.Get().transform.position.x > 8490)
					{
						tempInt--;
					}
				}
				
			}
			
			framesSinceStart++;
			if(framesSinceStart%tempInt == 0)
			{
				frame++;
				if( frame >= spritesX)
				{
					frame = 0;
				}
				if(state == 0)
				{
					if(frame == 3)
						MyLittleSoundManager.Get().HoofBeat(1);
					if(frame == 0)
						MyLittleSoundManager.Get().HoofBeat(2);
					if(frame == 1)
						MyLittleSoundManager.Get().HoofBeat(3);
				}
			}
			
			if(state == 0)
			{
				
				sprite.material.mainTextureOffset = new Vector2((float)frame/(float)spritesX, 0);
			
				if(frame == 0)
				{
					hornPos = new Vector3(13, 7.5f, -1);
				}
				else if(frame == 1)
				{
					hornPos = new Vector3(13, 7, -1);
				}
				else if(frame == 2)
				{
					hornPos = new Vector3(12.5f, 7, -1);
				}
				else if(frame == 3)
				{
					hornPos = new Vector3(12.5f, 7.5f, -1);
				}
			}
			else if(state == 1)
			{
				sprite.material.mainTextureOffset = new Vector2(0, 1f/(float)spritesY);
				hornPos = new Vector3(8, 10.5f, -1);
			}
			else if(state == 2)
			{
				sprite.material.mainTextureOffset = new Vector2(1/(float)spritesX, 1f/(float)spritesY);
				hornPos = new Vector3(13, 9, -1);
			}
			else if(state == 3)
			{
				sprite.material.mainTextureOffset = new Vector2(2/(float)spritesX, 1f/(float)spritesY);
				hornPos = new Vector3(13f, 7, -1);
			}
		}
		else
		{
			// flight animation goes here
			framesSinceStart++;
			if(framesSinceStart%6 == 0)
			{
				frame++;
				if( frame >= spritesX)
				{
					frame = 0;
				}
				
				if(frame == 3)
				{
					MyLittleSoundManager.Get().WingFlap();
				}
			}
			sprite.material.mainTextureOffset = new Vector2((float)frame/(float)spritesX, 2f/spritesY);
			
			if(frame == 0)
				{
					hornPos = new Vector3(13, 7.5f, -1);
				}
				else if(frame == 1)
				{
					hornPos = new Vector3(13, 6.5f, -1);
				}
				else if(frame == 2)
				{
					hornPos = new Vector3(13, 7.5f, -1);
				}
				else if(frame == 3)
				{
					hornPos = new Vector3(13, 8.5f, -1);
				}
			
		}
		// handle magic animation
		if(chargeLevel == 0)
		{
			//hide me
			magicSprite.gameObject.SetActiveRecursively(false);
		}
		else
		{
			magicSprite.gameObject.SetActiveRecursively(true);
			// ponies
			magicSprite.transform.localPosition = hornPos;
			magicSprite.material.mainTextureOffset = new Vector2(frame/4f, (chargeLevel-1)/4f);
		}
		
    }
}