﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour
{
	public int momentumCapY1 = 9;
	public int momentumCapY2 = 4;
	public float momentumCapX = 2.5f;
	public int edgeLag = 0;
	public float sensitivityX = 0.01f;
	public float airJumpPower = 6;
	public int airJumpDuration = 15;
	public float dashPower = 6;
	public int dashDuration = 5;
	public float divePower = 9;
	
	// state in wich the player hurt enemies.
	
	static private PlayerController instance;
	
	private Vector3 startingPos;
	private float verticalMomentum = 0;
	private int verticalMove = 0;
	private CharacterController charcont;
	private float horisontalMomentum = 2;
//	private int tickSinceStart = 0;
	private bool gravity = true;
	private int jump = 10;
//	private int timesinceleftground = 0;
	private Animator myAnimator;
	private bool charge = false;
	private ChargeNode chargeTarget;
	private int framesToReachChargeTarget = 0;
	
	private bool canAirJump = false;
	private int airJumpTimer = 0;
	
	private bool canDash = false;
	private int dashTimer = 0;
	
	private bool dive = false;
	private int slowDown = 0;
	private bool overrideDiveSpeed = false;
	private int attackModeDuration = 0;
	
	public bool Attacking
	{
		set{}
		get
		{
			if(overrideDiveSpeed) return true;
			if(attackModeDuration > 0) return true;
			return false;
		}
	}
	public float CameraMovementModifier
	{
		set{}
		get
		{
			// if dashing +1
			if(dashTimer > 0)
			{
				return 1;
			}
			// if slowing down -1
			if(slowDown > 0)
			{
				return -1;
			}
			// else nothing
			return 0;
		}
	}
	
	/// <summary>
	/// 1 is Charge, 2 is jump, 3 is dash, 4 is dive, 5 is slowDown
	/// </value>
	public int LastMove
	{
		private set;
		get;
	}

    void Awake ()
    {
		charcont = GetComponent<CharacterController>();
		myAnimator = GetComponent<Animator>();
		instance = this;
		startingPos = transform.position;
    }
	
	public void Reset()
	{
		transform.position = startingPos;
	}
	
	static public PlayerController Get()
	{
		return instance;
	}
		
   
    void Update ()
    {
		/*if(Input.GetButton("Up") && Input.GetButtonDown("Down"))
		{
			// dont move
			verticalMove = 0;
		}
		else if(Input.GetButton("Up"))
		{
			// move up
			verticalMove = 1;
		}
		else if(Input.GetButton("Down"))
		{
			// move down
			verticalMove = -1;
		}
		else
		{
			verticalMove = 0;
		}*/
		
		if(Input.GetButtonDown("Up"))
		{
			jump = 0;
		}
    }
	
	void FixedUpdate()
	{
		// player should gravitate towards camera.x -80
		
		float inputX = 0; // unless its not
		float inputY = 0;
		
		inputY = GenericInfo.Get().speed * verticalMove;
		inputX += Input.GetAxis("Horisontal");
		horisontalMomentum += charcont.isGrounded ? inputX*sensitivityX : inputX*sensitivityX*0.3f;
		
		if(slowDown > 0)
		{
			horisontalMomentum /= 4;
			horisontalMomentum--;
			
			slowDown--;
			verticalMomentum /= 4;
			attackModeDuration--;
		}
		else if(dive)
		{
			verticalMomentum = (verticalMomentum < 0) ? verticalMomentum - divePower - Mathf.Abs(horisontalMomentum - GenericInfo.Get().speed) : -divePower - Mathf.Abs(horisontalMomentum - GenericInfo.Get().speed);
			overrideDiveSpeed = true;
			dive = false;
			attackModeDuration = 3;
		}
		else if(dashTimer > 0)
		{
			dashTimer--;
			if(dashTimer == 0)
			{
				horisontalMomentum = GenericInfo.Get().speed;
			}
			
			verticalMomentum = 0;
			horisontalMomentum = GenericInfo.Get().speed + dashPower;
			attackModeDuration = 4;
			
		}		
		else if(airJumpTimer > 0)
		{
			airJumpTimer--;
			
			attackModeDuration = 3;
			verticalMomentum = airJumpPower;
			horisontalMomentum = GenericInfo.Get().speed;
			
			if(airJumpTimer == 0)
			{
				verticalMomentum = 2;
			}
			
		}
		else if(!charge && !MyLittleLevelManager.Get().setupBossFight )
		{
			attackModeDuration--;
			if(charcont.isGrounded)
			{
				if(jump < 4)
				{
					verticalMomentum = 4;
				}
				else
				{
					verticalMomentum = 0;
				}
				edgeLag = 0;
				RecoverMoves();
				MyLittleHighScore.Get().ResetCombo();
				overrideDiveSpeed = false;
			}
			else
			{
				if(Input.GetButton("Up") && verticalMomentum > 0)
				{
					verticalMomentum += 0.1f;
					if(edgeLag < 4)
					{
						verticalMomentum += 0.4f;
					}
				}
				edgeLag++;
			}
			jump++;
			
			
			// adjust X to gravitate towards -80 if there is no input and on the ground
			if(charcont.isGrounded && Input.GetAxis("Horisontal") == 0)
			{
				float deltaX = Mathf.Abs(transform.position.x - (MyLittleCamera.Get().transform.position.x - 80));
				
				if(transform.position.x < MyLittleCamera.Get().transform.position.x - 80 && horisontalMomentum < GenericInfo.Get().speed + 0.3f)
				{
					// try n catch up
					horisontalMomentum += 0.002f * deltaX;
				}
				else if(transform.position.x > MyLittleCamera.Get().transform.position.x - 70 && horisontalMomentum > GenericInfo.Get().speed - 0.2f)
				{
					horisontalMomentum -= 0.002f * deltaX;
				}
			}
			
			// Gravity!
			if(gravity)
				{
				inputY -= GenericInfo.Get().gravity;
			}
			verticalMomentum += inputY;
			if(verticalMomentum < (MyLittleLevelManager.Get().Level2 ? -momentumCapY2 : -momentumCapY1) &! overrideDiveSpeed)
			{
				verticalMomentum = (MyLittleLevelManager.Get().Level2 ? -momentumCapY2 : -momentumCapY1);
			}
			else if(verticalMomentum > momentumCapY1)
			{
				verticalMomentum = momentumCapY1;
			}
			
			if(horisontalMomentum < GenericInfo.Get().speed-momentumCapX)
			{
				horisontalMomentum =  GenericInfo.Get().speed-momentumCapX;
			}
			else if(horisontalMomentum > momentumCapX+GenericInfo.Get().speed)
			{
				horisontalMomentum = momentumCapX+GenericInfo.Get().speed;
			}
		}
		
		else if(charge)// charging
		{
			// tween to charge position in 10(?) frames
			// get movement required to reach the target
			Vector3 reqMove = chargeTarget.transform.position - transform.position;
			// divide with number of frames needed to reach it
			if(framesToReachChargeTarget != 0)
			{
				reqMove /= framesToReachChargeTarget;
			}
			
			// translate it to outside the block
			horisontalMomentum = reqMove.x;
			verticalMomentum = reqMove.y;
			
			// set charging to false if there
			framesToReachChargeTarget--;
			if(framesToReachChargeTarget < 1)
			{
				charge = false;
				// if horisotnal momentum is more than speed+2, reduce it to speed+2 and reduce the vert momentum with the same. 
				if(horisontalMomentum > GenericInfo.Get().speed+momentumCapX)
				{
					float multiplier = (GenericInfo.Get().speed+momentumCapX)/horisontalMomentum;
					horisontalMomentum *= multiplier;
					verticalMomentum *= multiplier;
				}
			}
			overrideDiveSpeed = false;
			attackModeDuration = 3;
		}
		else if(MyLittleLevelManager.Get().bossFight)
		{
			// BOSS!!! Fight controls!!!   its a smup
			verticalMomentum = 0;
			horisontalMomentum = 0;
			if(Input.GetButton("Up"))
			{
				verticalMomentum = 2.5f;
			}
			if(Input.GetButton("Down"))
			{
				verticalMomentum = -2.5f;
			}
			
			
			RecoverMoves();
			
			// catch offscreen
			if(transform.position.y < -85) // bottom
			{
				// force upwards movement
				if(verticalMomentum <= 0)
				{
					verticalMomentum = +1f;
				}
			}
			if(transform.position.y > 85) // top
			{
				// force downwards movement
				if(verticalMomentum >= 0)
				{
					verticalMomentum = -1f;
				}
			}
			
			
			// assign the move
			horisontalMomentum = Input.GetAxis("Horisontal")*3.5f + GenericInfo.Get().speed;
		}
		else if(MyLittleLevelManager.Get().setupBossFight)
		{
			// say hello to gravity
			verticalMomentum -= GenericInfo.Get().gravity;
		}
		
		//catch before going offscreen
		if(transform.position.x < MyLittleCamera.Get().transform.position.x - 122)
		{
			// hit the left edge, dont allow it to go futher that way.
			if(horisontalMomentum < GenericInfo.Get().speed)
			horisontalMomentum = GenericInfo.Get().speed + 0.1f;
		}
		else if(transform.position.x > MyLittleCamera.Get().transform.position.x + 112)
		{
			// hit the right edge
			if(horisontalMomentum > GenericInfo.Get().speed)
				horisontalMomentum = GenericInfo.Get().speed -0.1f;
		}
		
		charcont.Move(new Vector3(horisontalMomentum, verticalMomentum, 0));
		
		
		// figure out animations
		if(charcont.isGrounded)
		{
			myAnimator.state = 0;
		}
		else
		{
			if(verticalMomentum < -1) // acending
			{
				myAnimator.state = 3;
			}
			else if(verticalMomentum > 1) // falling
			{
				myAnimator.state = 1;
			}
			else // neutral
			{
				myAnimator.state = 2;
			}
		}
		
		// if outside the screen, reset game
		if(transform.position.y < -180 || transform.position.x < MyLittleCamera.Get().transform.position.x - 150)
		{
			if(!MyLittleLevelManager.Get().setupBossFight)
			{
				MyLittleResetter.Reset();
				MyLittleSoundManager.Get().Crash();
			}
			else
			{
				MyLittleLevelManager.Get().StartBossFight();
			}
		}
	}
	
	public void Charge(ChargeNode target)
	{
		if(!Attacking)
		{
			charge = true;
			chargeTarget = target;
			framesToReachChargeTarget = 10;
			LastMove = 1;
		}
	}
	
	public void AirJump()
	{
		if(canAirJump &! charge && (dashTimer < 1))
		{
			canAirJump = false;
			airJumpTimer = airJumpDuration;
			LastMove = 2;
			overrideDiveSpeed = false;
		}
	}
	
	public void Dash()
	{
		if(canDash &! charge && (airJumpTimer < 1))
		{
			canDash = false;
			dashTimer = dashDuration;
			LastMove = 3;
			overrideDiveSpeed = false;
		}
	}
	public void Dive()
	{
		dive = true;
		LastMove = 4;
	}
	
	public void SlowDown()
	{
		slowDown = 4;
		LastMove = 5;
		overrideDiveSpeed = false;
	}
	
	public void RecoverMoves()
	{
		canDash = true;
		canAirJump = true;
	}
	
	public void Shoot(int power)
	{
		// write me!
		
	}
}

/*
 * make it so that the camere moves instead, and the player is keeping up with the camera!!!!
 * */